/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package functions;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import javabot_client.javabot_client;


public class ButiaFunctions {
    
    private javabot_client client;
    
    public ButiaFunctions(){
        // default: HOST localhost , PORT 2009
        this.client = new javabot_client();
    }
    
    public ButiaFunctions(javabot_client client){
        this.client = client;
    }
            
    public boolean isPresent(String module_name) {
        String[] modules = client.getModulesList();
        return Arrays.asList(modules).contains(module_name);

    }

    public String loopBack(String data, int board) {
        /* 
         LoopBack command: send data to the board and get the result. If all is ok
         the return must be exactly of the data parameter
         */
        String[] Params = new String[]{data};
        return client.callModule("lback", board, 0, "send", Params);
    }
    

    /*############################## Movement calls ##############################*/
    public int set2MotorSpeed(int leftSense, int leftSpeed, int rightSense, int rightSpeed, int board) {
        /*Set the speed of 2 motors. The sense is 0 or 1, and the speed is
         between 0 and 1023*/

        String[] params = new String[]{String.valueOf(leftSense), String.valueOf(leftSpeed), String.valueOf(rightSense), String.valueOf(rightSpeed)};
        return Integer.parseInt(client.callModule("motors", board, 0, "setvel2mtr", params));
    }

    public int setMotorSpeed(int idMotor, int sense, int speed, int board) {
        /*
         Set the speed of one motor. idMotor = 0 for left motor and 1 for the
         right motor. The sense is 0 or 1, and the speed is between 0 and 1023
         */

        String[] params = new String[]{String.valueOf(idMotor), String.valueOf(sense), String.valueOf(speed)};
        return Integer.parseInt(client.callModule("motors", board, 0, "setvelmtr", params));
    }

    public int getMotorType(int board) {
        /*
         If AX-12 motors present returns 1. If there are a shield "cc" returns 2 
         */
        return Integer.parseInt(client.callModule("motors", board, 0, "getType", null));
    }
    
    
    /*##################### Operations for ax.lua driver #########################*/

    public int writeInfo(int idMotor,int regstart,int value,int board){
        /*
        Writes the motor: idMotor in the registry: regstart with value: value
        */
        String[] params = new String[]{String.valueOf(idMotor), String.valueOf(regstart), String.valueOf(value)};
        return Integer.parseInt(client.callModule("ax", board, 0, "writeInfo", params));

    };
            
    public int readInfo(int idMotor, int regstart, int length, int board){
        /*
        Reads the motor: idMotor in the registry: regstart
        */
        String[] params = new String[]{String.valueOf(idMotor), String.valueOf(regstart), String.valueOf(length)};
        return Integer.parseInt(client.callModule("ax", board, 0, "writeInfo", params));

    };


    public int sendPacket(int idMotor, int[] msg, int board){
        /*
        Send a raw packet to ax module
        */
        List<String> list = new ArrayList<>();
        list.add(String.valueOf(idMotor));
        for (int i : msg){
            list.add(String.valueOf(i));
        }
        String[] params = new String[ list.size() ];
        list.toArray( params );
        return Integer.parseInt(client.callModule("ax", board, 0, "sendPacket", params));

    };
            
    public int wheelMode(int idMotor, int board){
        /*
        Sets the motor: idMotor in wheel mode (continuos rotation)
        */
        String[] params = new String[]{String.valueOf(idMotor)};
        return Integer.parseInt(client.callModule("ax", board, 0, "wheelMode", params));

    };
    
    public int jointMode(int idMotor,int min, int max, int board){
        /*
        Sets the motor: idMotor in servo mode
        */
        String[] params = new String[]{String.valueOf(idMotor), String.valueOf(min), String.valueOf(max)};
        return Integer.parseInt(client.callModule("ax", board, 0, "jointMode", params));

    };
            
    public int setPosition(int idMotor,int pos, int board){
        /*
        Sets the position: pos of the motor: idMotor
        */
        String[] params = new String[]{String.valueOf(idMotor), String.valueOf(pos)};
        return Integer.parseInt(client.callModule("ax", board, 0, "setPosition", params));

    };

    public int getPosition(int idMotor, int board){
        /*
        Gets the position of motor: idMotor
        */
        String[] params = new String[]{String.valueOf(idMotor)};
        return Integer.parseInt(client.callModule("ax", board, 0, "getPosition", params));

    };        
    
    public int setSpeed(int idMotor, int speed,  int board){
        /*
        Set the speed: speed to the motor: idMotor
        */
        String[] params = new String[]{String.valueOf(idMotor), String.valueOf(speed)};
        return Integer.parseInt(client.callModule("ax", board, 0, "setSpeed", params));

    };  
    
    
    /*############################### General calls ##############################*/

    public float getBatteryCharge(int board) {
        /*
         Gets the battery level charge
         */
        return Float.parseFloat(client.callModule("butia", board, 0, "getVolt", null));
    }
            
    public int getVersion(int board) {
        /*
         Gets the version of Butiá module. 22 for new version
         */
        return Integer.parseInt(client.callModule("butia", board, 0, "getVersion", null));
    }
    
    public int getFirmwareVersion(int board) {
        /*
         Gets the version of the Firmware
         */
        return Integer.parseInt(client.callModule("admin", board, 0, "getVersion", null));
    }
    

    /*############################### Sensors calls ###############################*/
    public int getButton(int port, int board) {
        /*
         Gets the value of the button connected in port
         */
        return Integer.parseInt(client.callModule("button", board, port, "getValue", null));
    }

    public int getLight(int port, int board) {
        /*
         Gets the value of the light sensor connected in port
         */
        return Integer.parseInt(client.callModule("light", board, port, "getValue", null));
    }

    public int getDistance(int port, int board) {
        /*
         Gets the value of the distance sensor connected in port
         */
        return Integer.parseInt(client.callModule("distanc", board, port, "getValue", null));
    }

    public int getGrey(int port, int board) {
        /*
         Gets the value of the gray sensor connected in port
         */
        return Integer.parseInt(client.callModule("grey", board, port, "getValue", null));
    }

    public float getResistance(int port, int board) {
        /*
         Gets the value of the resistance sensor connected in port
         */
        return Float.parseFloat(client.callModule("res", board, port, "getValue", null));
    }

    public float getVoltage(int port, int board){
        /*
         Gets the value of the voltage sensor connected in port
         */
        return Float.parseFloat(client.callModule("volt", board, port, "getValue", null));
    }

    public float getTemperature(int port, int board) {
        /*
         Gets the value of the temperature sensor connected in port
         */
        return Float.parseFloat(client.callModule("temp", board, port, "getValue", null));
    }

    public int setLed(int port, int on_off, int board) {
        /*
         Sets on or off the LED connected in port (0 is off, 1 is on)
         */
        return Integer.parseInt(client.callModule("led", board, port, "turn", new String[]{String.valueOf(on_off)}));
    }
    
    
    /*################################ Extras ################################*/

    public int setModeHack(int pin, int mode, int board){
        /*
        Sets the mode of hack pin. If mode 0 = output, mode 1 = input
        */
        String[] msg = new String[]{String.valueOf(pin),String.valueOf(mode)};
        return Integer.parseInt(client.callModule("hackp", board, 0, "setMode", msg));
    }
    
    public int getModeHack(int pin, int board){
        /*
        Get the mode of hack pin. If mode 0 = output, mode 1 = input
        */
        return Integer.parseInt(client.callModule("hackp", board, 0, "getMode", new String[]{String.valueOf(pin)}));
    }
    
    public int setHack(int pin, int value, int board){
        /*
        Sets the value of hack pin configured as output. Value is 0 or 1
        */
        String[] msg = new String[]{String.valueOf(pin),String.valueOf(value)};
        return Integer.parseInt(client.callModule("hackp", board, 0, "write", msg));
    }
    
    public int getHack(int pin, int board){
        /*
        Gets the value of hack pin configured as input. Returns 0 or 1
        */
        return Integer.parseInt(client.callModule("hackp", board, 0, "read", new String[]{String.valueOf(pin)}));
    }
    
    /*############################# Generic modules #############################*/

    public int getModuleA(int port, int board){
        /*
        Gets the value of the generic sensor A connected in port
        */
        return Integer.parseInt(client.callModule("modSenA", board, port, "getValue", null));
    }
    
    public int getModuleB(int port, int board){
        /*
        Gets the value of the generic sensor B connected in port
        */
        return Integer.parseInt(client.callModule("modSenB", board, port, "getValue", null));
    }
    
    public int getModuleC(int port, int board){
        /*
        Gets the value of the generic sensor C connected in port
        */
        return Integer.parseInt(client.callModule("modSenC", board, port, "getValue", null));
    }
    
    public int setModuleA(int port, int on_off, int board){
        /*
        Sets on or off the generic actuator module A
        */
        return Integer.parseInt(client.callModule("modActA", board, port, "turn", new String[]{String.valueOf(on_off)}));
    }
    
    public int setModuleB(int port, int on_off, int board){
        /*
        Sets on or off the generic actuator module B
        */
        return Integer.parseInt(client.callModule("modActB", board, port, "turn", new String[]{String.valueOf(on_off)}));
    }
    
    public int setModuleC(int port, int on_off, int board){
        /*
        Sets on or off the generic actuator module C
        */
        return Integer.parseInt(client.callModule("modActC", board, port, "turn", new String[]{String.valueOf(on_off)}));
    }
}
